@@ -16,13 +16,11 @@
 # You should have received a copy of the GNU General Public License
 # along with GWpy.  If not, see <http://www.gnu.org/licenses/>.
 
-"""This module provides plotting utilities for visualising GW data
-
-The standard data types (`TimeSeries`, `Table`, `DataQualityFlag`, ...) can
-all be easily visualised using the relevant plotting objects, with
-many configurable parameters both interactive, and in saving to disk.
+"""DEPRECATED - this module has been replaced by :mod:`gwpy.plot`
 """
 
+import warnings
+
 from matplotlib import pyplot
 
 # utilities
@@ -51,3 +49,7 @@ def figure(*args, **kwargs):  # pylint: disable=missing-docstring
     kwargs.setdefault('FigureClass', Plot)
     return pyplot.figure(*args, **kwargs)
 figure.__doc__ = pyplot.figure.__doc__
+
+
+warnings.warn('this module was replaced by `gwpy.plot` and will be '
+              'removed in a future release', DeprecationWarning)
